/*
   program : SheppLogan, recon, recon2img

   $Source: /home/yee/C/RCS/ImageHeader.c,v $
   $Author: yee $
   $Date: 1999/02/10 16:38:05 $
   $Revision: 3.8 $

   This file contains the code to handle the header for reconstruction data.

   Parameters : none

   Assumptions : none known

   Bugs : none known

   To Do :
     1) Convert to ACR-NEMA file format
     2) allow reading/writing functions to dynamically adjust for the size of
        each number (i.e. single vs. double precision).
     3) Code currently dependent upon system sizes for double, long.
     4) fix conflicting types for success code.

   WARNING : this code takes advantage of the returned value of fread and
             fwrite to be used as a success code!

   In the interest of simplicity in maintaining this code, the code
   contains inefficiencies in that there are multiple tests for read
   and write mode.  This was done to allow the sharing of as much code
   as possible between read/write modes.  Changes to the file format
   can be made simply to both modes by a single function call.
   
   $Log: ImageHeader.c,v $
   Revision 3.8  1999/02/10 16:38:05  yee
   *** empty log message ***

   Revision 3.7  1998/09/01 23:09:59  yee
   documentation fix.

   Revision 3.6  1998/09/01 22:02:02  yee
   documentation fix.

   Revision 3.5  1998/08/22 15:37:39  yee
   all geometries work!

   Copyright 1997, 1998 by Conway Yee.  All rights reserved.
 */

char ImageHeader_[]="$Id: ImageHeader.c,v 3.8 1999/02/10 16:38:05 yee Exp $";

#include <netinet/in.h>
#include <stdio.h>
#include <errno.h>

#include "codes.h"
#include "style.h"
#include "ImageHeader.h"

PRIVATE CODE iostring(HeaderString *,size_t (*)(),FILE *);
PRIVATE CODE iodouble(double *,size_t (*)(),FILE *);
PRIVATE CODE iolong(int *,size_t (*)(),FILE *);
 
/*****************************************************************************/
/*****************************************************************************/

/* this function handles the input/output of the image header */

PUBLIC CODE ioImageHeader
(
 ImageHeader *h,   /* ImageHeader to be input/output */
 size_t (*IO)(),   /* input vs output mode */
 FILE *fptr        /* file pointer to be input/output */
)
{
  CODE success=SUCCESS;       /* success code */

  {
    int magic=IMAGE_MAGIC;

    if(!iolong(&magic,IO,fptr))         /* magic number */
      success=FAILURE;
    if(success&&(IO==fread))
      if(magic!=IMAGE_MAGIC) {
        perror("ioImageHeader : illegal file format");
        success=FAILURE;
      }
  }

  if(success)
    if(!iolong(&(h->fsize),IO,fptr))    /* sizeof(float) */
      success=FAILURE;

  if(success)
    if(!iostring(&(h->name),IO,fptr))   /* patient name */
      success=FAILURE;
  if(success)
    if(!iostring(&(h->mr),IO,fptr))     /* patient medical record number */
      success=FAILURE;
  if(success)
    if(!iostring(&(h->loc),IO,fptr))    /* patient location */
    success=FAILURE;

  if(success)
    if(!iolong(&(h->type),IO,fptr))     /* image type */
      success=FAILURE;

  if(success)
    if(!iodouble(&(h->D),IO,fptr)) /* dist bet focal point & rotation center */
      success=FAILURE;

  if(success)
    if(!iolong(&(h->ndim),IO,fptr))     /* image dimensionality */
      success=FAILURE;
 
  {
    int i;    /* counter */
    int top;  /* determines the number of int/float to transmit */
    
    for(top=2*h->ndim,i=0;i<top;i++)           /* write image dimensions */
      if(success)
        if(!iolong(&(h->lim[i]),IO,fptr))
          success=FAILURE;

    for(top=h->ndim,i=0;i<top;i++)             /* write image dimensions */
      if(success) {
        if(!iolong(&(h->dim[i]),IO,fptr))
          success=FAILURE;
        if(h->dim[i]!=(h->lim[2*i+1]-h->lim[2*i]+1)) {
          success=FAILURE;
          perror("ioImageHeader : corrupt image header ");
        }
      }

    for(top=h->ndim,i=0;i<top;i++)             /* write distance */
      if(success)
        if(!iodouble(&(h->tau[i]),IO,fptr))
          success=FAILURE;
  }

  return(success);
}

/*****************************************************************************/
/***************** primitives used in ioImageHeader() ************************/
/*****************************************************************************/

/* this function handles input/output of the structures HeaderString */

PRIVATE CODE iostring
(
 HeaderString *string,           /* string to be read/written */
 size_t (*IO)(),                 /* fread() vs. fwrite() */
 FILE *fptr                      /* file to be read/written */
)
{
  int num;                      /* number to be sent/received */
  size_t success;               /* success code */

  if(IO==fwrite) {
    num=htonl(string->size);
    success=fwrite(&num,sizeof(int),1,fptr);
  } else {
    success=fread(&num,sizeof(int),1,fptr);
    string->size=ntohl(num);
    string->s=realloc(string->s,(string->size+1)*sizeof(char));
    string->s[string->size]='\0';
  }
  if(success) 
    return(IO(string->s,string->size,1,fptr));
  else
    return(FAILURE);
}

/*****************************************************************************/

/* this function handles input/output of doubles */

PRIVATE CODE iodouble
(
 double *num,             /* double to be read/written */
 size_t (*IO)(),          /* fread() vs. fwrite() */
 FILE *fptr               /* file to be read/written */
)
{
  return((*IO)(num,sizeof(double),1,fptr));
}

/*****************************************************************************/

/* this function handles input/output of integers */

PRIVATE CODE iolong
(
 int *num,                /* int to be read/written */
 size_t (*IO)(),          /* fread() vs. fwrite() */
 FILE *fptr               /* file to be read/written */
)
{
  size_t success;         /* success code */

  if(IO==fread) {
    success=fread(num,sizeof(int),1,fptr);
    *num=ntohl(*num);
  } else {
    int nmbr; /* dummy variable */

    nmbr=htonl(*num);
    success=fwrite(&nmbr,sizeof(int),1,fptr);
  }
  return(success);
}
